package np.something.controllers;

import np.something.DTO.CommentDto;
import np.something.model.Comment;
import np.something.services.*;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/comment")
public class CommentController {
    private final CommentService commentService;
    private final CustomerService customerService;
    private final PostService postService;

    public CommentController(CommentService commentService, CustomerService customerService, PostService postService) {
        this.commentService = commentService;
        this.customerService = customerService;
        this.postService = postService;
    }

    @GetMapping("/{id}")
    public CommentDto getComment(@PathVariable Long id) {
        return new CommentDto(commentService.findComment(id));
    }

    @GetMapping
    public List<CommentDto> getComments() {
        return commentService.findAllComments().stream()
                .map(CommentDto::new)
                .toList();
    }

    @PostMapping
    public CommentDto createComment(@RequestParam("text") String text, @RequestParam("ownerId") Long ownerId, @RequestParam("postId") Long postId){
        final Comment comment = commentService.addComment(customerService.findCustomer(ownerId), postService.findPost(postId), text);
        return new CommentDto(comment);
    }

    @PutMapping("/{id}")
    public CommentDto updateComment(@RequestParam("text") String text, @PathVariable Long id) {
        return new CommentDto(commentService.updateComment(id, text));
    }

    @DeleteMapping("/{id}")
    public CommentDto deleteComment(@PathVariable Long id) {
        return new CommentDto(commentService.deleteComment(id));
    }

    @DeleteMapping
    public void deleteAllComments(){
        commentService.deleteAllComments();
    }
}
