package np.something;

import np.something.model.*;
import np.something.services.CommentService;
import np.something.services.CustomerService;
import np.something.services.PostService;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

@SpringBootTest
public class SocialNetworkTest {
    @Autowired
    CustomerService customerService;

    @Autowired
    CommentService commentService;

    @Autowired
    PostService postService;

    @Test
    void testCustomers() {
        commentService.deleteAllComments();
        postService.deleteAllPosts();
        customerService.deleteAllCustomers();

        Customer c1 = customerService.addCustomer("first", "1");
        Customer c2 = customerService.addCustomer("second", "2");
        Customer c3 = customerService.addCustomer("third", "3");

        Assertions.assertEquals("first", c1.getUsername());
        Assertions.assertEquals("second", c2.getUsername());
        Assertions.assertEquals("third", c3.getUsername());

        Assertions.assertEquals(c1, customerService.findCustomer(c1.getId()));

        customerService.deleteCustomer(c2.getId());

        Assertions.assertEquals(2, customerService.findAllCustomers().size());

        Customer c4 = customerService.updateCustomer(c3.getId(), "fourth", "4");

        Assertions.assertNotEquals(c3.getUsername(), c4.getUsername());
        Assertions.assertNotEquals(c3.getPassword(), c4.getPassword());

        commentService.deleteAllComments();
        postService.deleteAllPosts();
        customerService.deleteAllCustomers();
    }

    @Test
    void testPost() {
        commentService.deleteAllComments();
        postService.deleteAllPosts();
        customerService.deleteAllCustomers();

        Customer c1 = customerService.addCustomer("first", "1");
        Customer c2 = customerService.addCustomer("second", "2");

        Post p1 = postService.addPost(c1, "first title", "nonsense");
        Post p2 = postService.addPost(c2, "second title", "ordinal");

        Assertions.assertEquals(2, postService.findAllPosts().size());

        Assertions.assertEquals(p1.getCustomer(), c1);
        Assertions.assertEquals(p2.getCustomer(), c2);

        Assertions.assertEquals(c1.getPosts().get(0), p1);
        Assertions.assertEquals(c2.getPosts().get(0), p2);

        Assertions.assertEquals(p1, postService.findPost(p1.getId()));
        Assertions.assertEquals(p2, postService.findPost(p2.getId()));

        Post p3 = postService.addPost(c1, "asdf", "asd");
        postService.deletePost(p1.getId());
        Assertions.assertEquals(1, customerService.findCustomer(c1.getId()).getPosts().size());

        Post p4 = postService.updatePost(p2.getId(), "third title", "wow");

        Assertions.assertNotEquals(p2.getTitle(), p4.getTitle());
        Assertions.assertNotEquals(p2.getContent(), p4.getContent());

        commentService.deleteAllComments();
        postService.deleteAllPosts();
        customerService.deleteAllCustomers();
    }

    @Test
    void testComment() {
        commentService.deleteAllComments();
        postService.deleteAllPosts();
        customerService.deleteAllCustomers();

        Customer c1 = customerService.addCustomer("first", "1");
        Customer c2 = customerService.addCustomer("second", "2");

        Post p1 = postService.addPost(c1, "first title", "nonsense");
        Post p2 = postService.addPost(c2, "second title", "ordinal");

        Assertions.assertEquals(2, postService.findAllPosts().size());

        Comment com1 = commentService.addComment(c1, p2, "What");
        Comment com2 = commentService.addComment(c2, p1, "How");

        Assertions.assertEquals(c1, p2.getComments().get(0).getCustomer());
        Assertions.assertEquals(c2, p1.getComments().get(0).getCustomer());

        Comment com3 = commentService.addComment(c1, p1, "Really");

        Assertions.assertEquals(com2, commentService.findComment(p1.getComments().get(0).getId()));

        Comment com4 = commentService.updateComment(com3.getId(), "Not really");

        Assertions.assertNotEquals(com3.getContent(), com4.getContent());
        Assertions.assertEquals(com3.getCustomer().getId(), com4.getCustomer().getId());

        commentService.deleteAllComments();
        postService.deleteAllPosts();
        customerService.deleteAllCustomers();
    }
}
